package MT::Plugin::CMSContext;
use strict;
use MT;
use MT::Plugin;
use MT::Tag;
use MT::Request;

our $VERSION = '0.7';

use base qw( MT::Plugin );

@MT::Plugin::CMSContext::ISA = qw( MT::Plugin );

my $plugin = new MT::Plugin::CMSContext( {
    name => 'CMSContext',
    description => '<MT_TRANS phrase=\'_PLUGIN_DESCRIPTION\'>',
    author_name => 'Alfasado, Inc.',
    author_link => 'http://alfasado.net/',
    version     => $VERSION,
    settings => new MT::PluginSettings ( [
        [ 'plugin_active', { Default => 1 } ], # old
        [ 'tpl_directory', { Default => '' } ],
        [ 'tpl_iphone', { Default => '' } ],
        [ 'tpl_docomo', { Default => '' } ],
        [ 'tpl_au', { Default => '' } ],
        [ 'tpl_softbank', { Default => '' } ],
    ] ),
    config_template => 'cmsctx_config.tmpl',
    blog_config_template => 'cmsctx_config.tmpl',
    l10n_class => 'CMSContext::L10N',
} );

MT->add_plugin( $plugin );

sub init_registry {
    my $plugin = shift;
    $plugin->registry( {
        tags => {
            block => {
                'CMSContext' => \&_hdlr_cms_ctx,
                'IfUserCan?' => \&_hdlr_if_can,
                'IfUserAgentIsiPhone?' => \&_hdlr_if_user_agent,
                'IfUserAgentIsDoCoMo?' => \&_hdlr_if_user_agent,
                'IfUserAgentIsAU?' => \&_hdlr_if_user_agent,
                'IfUserAgentIsSoftBank?' => \&_hdlr_if_user_agent,
                'IfUserAgentIsMobile?' => \&_hdlr_if_mobile,
                'IfUserAgentIsKeitai?' => \&_hdlr_if_keitai,
            },
            function => {
                'UserAgent' => \&_hdlr_user_agent,
            },
        },
        callbacks => {
            'MT::App::CMS::take_down'
                => \&_take_down,
            'MT::App::CMS::template_param'
                => \&_cb_tmpl,
        },
   } );
}

sub init_request {
    my $plugin = shift;
    my ( $app ) = @_;
    if ( $app->isa( 'MT::App::CMS' ) ) {
        my $ua = _get_ua( $app );
        my $tpl_directory;
        if ( $ua eq 'iPhone' ) {
            $tpl_directory = $plugin->get_config_value( 'tpl_iphone' );
        } elsif ( $ua eq 'SoftBank' ) {
            $tpl_directory = $plugin->get_config_value( 'tpl_softbank' );
        } elsif ( $ua eq 'au' ) {
            $tpl_directory = $plugin->get_config_value( 'tpl_au' );
        }
        if ( (! $tpl_directory ) && ( $ua eq 'Mozilla' ) ) {
            $tpl_directory = $plugin->get_config_value( 'tpl_directory' );
        }
        my $blog = $app->blog;
        if ( defined $blog ) {
            my $get_from = 'blog:'. $blog->id;
            my $blog_tpl_directory;
            if ( $ua eq 'iPhone' ) {
                $blog_tpl_directory = $plugin->get_config_value( 'tpl_iphone', $get_from );
            } elsif ( $ua eq 'SoftBank' ) {
                $blog_tpl_directory = $plugin->get_config_value( 'tpl_softbank', $get_from );
            } elsif ( $ua eq 'au' ) {
                $blog_tpl_directory = $plugin->get_config_value( 'tpl_au', $get_from );
            }
            if ( (! $blog_tpl_directory ) && ( $ua eq 'Mozilla' ) ) {
                $blog_tpl_directory = $plugin->get_config_value( 'tpl_directory', $get_from );
            }
            if ( $blog_tpl_directory ) {
                $tpl_directory = $blog_tpl_directory;
            }
        }
        if ( $tpl_directory ) {
            require MT::ConfigMgr;
            my $mgr = MT::ConfigMgr->instance;
            $mgr->define( 'AltTemplatePath' );
            my $plugin_tpldir = File::Spec->catdir( $plugin->path, 'tmpl', $tpl_directory );
            $app->config( 'AltTemplatePath', $plugin_tpldir );
        }
    }
}

sub _take_down {
    my ( $cb, $app ) = @_;
    $app->config( 'AltTemplatePath', File::Spec->catdir( $app->mt_dir, 'alt-tmpl' ) );
}

sub _cb_tmpl {
    my ( $cb, $app, $param, $tmpl ) = @_;
    my $q = $app->param;
    for my $key ( $q->param ) {
        $param->{ 'cms:' . $key } = $q->param( $key );
    }
}

sub _hdlr_cms_ctx {
    my ( $ctx, $args, $cond ) = @_;
    my $r = MT::Request->instance;
    my $app = MT->instance();
    my $author = $app->user;
    my $blog = $app->blog;
    my $q = $app->param;
    my $type = $q->param( '_type' );
    my $mode = $q->param( '__mode' );
    my $reedit = $q->param( 'reedit' );
    my $id = $q->param( 'id' );
    my $tokens = $ctx->stash( 'tokens' );
    my $builder = $ctx->stash( 'builder' );
    local $ctx->{ __stash }{ blog } = $blog;
    local $ctx->{ __stash }{ blog_id } = $blog->id if $blog;
    my $class = MT->model( $type );
    my $entry = $r->cache( 'cms_cache_entry' );
    unless ( defined $entry ) {
        $entry = $class->load( $id ) if ( ( $id ) && ( $mode eq 'view' ) 
                                                    && ( ( $type eq 'entry') || ( $type eq 'page') ) );
        $r->cache( 'cms_cache_entry', $entry ) if ( $entry );
    }
    $entry = $class->new if ( (! $id ) && ( $mode eq 'view' ) 
                                          && ( ( $type eq 'entry') || ( $type eq 'page') ) );
    if ( defined $entry ) {
        if ( $reedit ) {
            my $clumns = $entry->column_names;
            for my $key ( $q->param ) {
                if ( grep( /^$key$/, @$clumns ) ) {
                    $entry->$key( $q->param ( $key ) );
                } else {
                    if ( $key eq 'tags' ) {
                        my $tag_delim = chr( $author->entry_prefs->{ tag_delim } );
                        my @tags = MT::Tag->split( $tag_delim, $q->param ( $key ) );
                        $entry->set_tags( @tags );
                    }
                }
            }
        }
    }
    my $category = $r->cache( 'cms_cache_category' );
    unless ( defined $category ) {
        $category = $class->load( $id ) if ( ( $id ) && ( $mode eq 'view' )
                                          && ( ( $type eq 'category') || ( $type eq 'folder' ) ) );
        $r->cache( 'cms_cache_category', $category ) if ( $category );
    }
    my $user = $app->user;
    if ( ( $mode eq 'view' ) && ( ( $id ) && ( $type eq 'author') ) ) {
        $user = $r->cache( 'cms_cache_user' );
        unless ( defined $user ) {
            $user = MT::Author->load( $id );
            $r->cache( 'cms_cache_user', $user ) if ( $user );
        }
    }
    local $ctx->{ __stash }{ entry } = $entry if ( defined $entry );
    local $ctx->{ __stash }{ category } = $category if ( $type eq 'category' );
    local $ctx->{ __stash }{ folder } = $category if ( $type eq 'folder' );
    local $ctx->{ __stash }{ author } = $user;
    my $out = $builder->build( $ctx, $tokens, $cond );
    $out;
}

sub _hdlr_if_can {
    my ( $ctx, $args, $cond ) = @_;
    my $permission = $args->{ permission };
    return 0 unless $permission;
    $permission = 'can_' . $permission;
    my $app = MT->instance;
    my $user = $app->user;
    my $blog = $ctx->stash( 'blog' );
    return 0 unless defined $user;
    my $perm = $user->is_superuser;
    unless ( $perm ) {
        if ( $blog ) {
            $perm = $user->permissions( $blog->id )->$permission;
        }
    }
    return $perm;
}

sub _hdlr_user_agent {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    return $app->get_header( 'User-Agent' );
}

sub _hdlr_if_user_agent {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    my $tag = lc $ctx->stash( 'tag' );
    $tag =~ s/^ifuseragentis//i;
    my $ua = lc ( _get_ua( $app ) );
    if ( $tag eq $ua ) {
        return 1;
    }
    return 0;
}

sub _hdlr_if_mobile {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    my $ua = _get_ua( $app );
    if ( $ua ne 'Mozilla' ) {
        return 1;
    }
    return 0;
}

sub _hdlr_if_keitai {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    my $ua = _get_ua( $app );
    if ( ( $ua ne 'Mozilla' ) && ( $ua ne 'iPhone' ) ) {
        return 1;
    }
    return 0;
}

sub _get_ua {
    my $app = shift;
    my $r = MT::Request->instance;
    my $cache = $r->cache( 'cms_cache_user_agent' );
    return $cache if $cache;
    my $agent;
    my $ua = $app->get_header( 'User-Agent' );
    if ( ( $ua =~ /iPhone/ ) || ( $ua =~ /iPod/ ) ) {
        $r->cache( 'cms_cache_user_agent', 'iPhone' );
        return 'iPhone';
    }
    my @browswer = split( m!/!, $ua );
    $ua = $browswer[0];
    if ( $ua eq 'DoCoMo' ) {
        $r->cache( 'cms_cache_user_agent', 'DoCoMo' );
        return 'DoCoMo';
    }
    if ( ( $ua eq 'SoftBank' ) || ( $ua eq 'Vodafone' ) ) {
        $r->cache( 'cms_cache_user_agent', 'SoftBank' );
        return 'SoftBank';
    }
    if ( $ua =~ /UP\.Browser/ ) {
        $r->cache( 'cms_cache_user_agent', 'au' );
        return 'au';
    }
#     if ( $ua =~ /^Nokia/ ) {
#         $r->cache( 'cms_cache_user_agent', 'Nokia' );
#         return 'Nokia';
#     }
    $r->cache( 'cms_cache_user_agent', 'Mozilla' );
    return 'Mozilla';
}

1;